import sys
import re


def main():
    input_data = sys.stdin.read()
    table_header = '\n\n| Test Description | Times | Outliers |\n| --- | --- | --- |\n'
    table_body = ''

    benchmarks = re.findall(r'^(.+)\n?\s+time:\s+\[(.+)\]([\s\S]+?)Found (\d+) outliers',
                            input_data, re.MULTILINE)

    for benchmark in benchmarks:
        test_description = benchmark[0]
        time = benchmark[1]
        outliers = benchmark[3]

        table_body += f'| {test_description} | {time} | {outliers} |\n'

    markdown_table = table_header + table_body
    print(markdown_table)


if __name__ == '__main__':
    main()



"""
 * Hifitime, part of the Nyx Space tools
 * Copyright (C) 2022 Christopher Rabotin <christopher.rabotin@gmail.com> et al. (cf. AUTHORS.md)
 * This Source Code Form is subject to the terms of the Apache
 * v. 2.0. If a copy of the Apache License was not distributed with this
 * file, You can obtain one at https://www.apache.org/licenses/LICENSE-2.0.
 *
 * Documentation: https://nyxspace.com/
"""

from hifitime import Duration, Epoch, TimeSeries, TimeScale, Unit

if __name__ == "__main__":
    # All of the functions available in Rust are also available in Python.
    # However, functions that initialize a new Epoch or Duration had to be renamed: `Epoch::from_blah` becomes `Epoch.init_from_blah` in Python.

    # Let's start by getting the system time.
    e = Epoch.system_now()
    # By default, it'll be printed in UTC
    print(f"UTC epoch: {e}")
    # But we can also print it in TAI
    print(f"TAI epoch: {e.to_gregorian_tai()}")

    # Or we can print it in a specific time system
    print(f"ET epoch: {e.to_string_gregorian(TimeScale.ET)}")
    print(f"TDB epoch: {e.to_string_gregorian(TimeScale.TDB)}")

    # Hifitime mainly allows for nanosecond precision of durations for 64 centuries (centered on J1900).
    print(f"min negative = {Duration.min_negative()}")
    print(f"min positive = {Duration.min_positive()}")

    # And more importantly, it does not suffer from rounding issues, even when the duration are very large.
    print(f"Max duration: {Duration.max()}")  # 1196851200 days
    print(f"Nanosecond precision: {Duration.max() - Unit.Nanosecond * 1.0}")
    assert f"{Unit.Day * 1.2}" == "1 days 4 h 48 min"
    assert f"{Unit.Day * 1.200001598974}" == "1 days 4 h 48 min 138 ms 151 μs 353 ns"

    # It also saturates the duration
    print(f"Saturated add: {Duration.max() + Unit.Day * 1.0}")

    # You can also get all of the epochs between two different epochs at a specific step size.
    # This is like numpy's `linspace` with high fidelity durations
    time_series = TimeSeries(
        Epoch.system_now(),
        Epoch.system_now() + Unit.Day * 0.3,
        Unit.Hour * 0.5,
        inclusive=True,
    )
    print(time_series)
    for num, epoch in enumerate(time_series):
        print(f"#{num}:\t{epoch}")

    e1 = Epoch.system_now()
    e3 = e1 + Unit.Day * 1.5998
    epoch_delta = e3.timedelta(e1)
    assert (
        epoch_delta
        == Unit.Day * 1 + Unit.Hour * 14 + Unit.Minute * 23 + Unit.Second * 42.720
    )
    print(epoch_delta)



import sys
import re


def main():
    input_data = sys.stdin.read()
    table_header = "\n\n| Benchmark | Instructions | L1 Accesses | L2 Accesses | RAM Accesses | Estimated Cycles |\n| --- | --- | --- | --- | --- | --- |\n"
    table_body = ""

    benchmarks = input_data.split("\n\n")

    for benchmark in benchmarks:
        splt = [s.strip() for s in benchmark.split("\n")]
        if len(splt) != 6:
            continue
        name = splt[0]
        instructions = re.search(r"Instructions:\s+(\d+)", splt[1]).group(1)
        l1_accesses = re.search(r"L1 Accesses:\s+(\d+)", splt[2]).group(1)
        l2_accesses = re.search(r"L2 Accesses:\s+(\d+)", splt[3]).group(1)
        ram_accesses = re.search(r"RAM Accesses:\s+(\d+)", splt[4]).group(1)
        estimated_cycles = re.search(r"Estimated Cycles:\s+(\d+)", splt[5]).group(1)

        table_body += f"| {name} | {instructions} | {l1_accesses} | {l2_accesses} | {ram_accesses} | {estimated_cycles} |\n"

    markdown_table = table_header + table_body
    print(markdown_table)


if __name__ == "__main__":
    main()



"""
 * Hifitime, part of the Nyx Space tools
 * Copyright (C) 2022 Christopher Rabotin <christopher.rabotin@gmail.com> et al. (cf. AUTHORS.md)
 * This Source Code Form is subject to the terms of the Apache
 * v. 2.0. If a copy of the Apache License was not distributed with this
 * file, You can obtain one at https://www.apache.org/licenses/LICENSE-2.0.
 *
 * Documentation: https://nyxspace.com/
"""

try:
    import plotly.express as px
except ImportError:
    print("\nThis script requires `plotly` (pip install plotly)\n")

try:
    import pandas as pd
except ImportError:
    print("\nThis script requires `pandas` (pip install pandas)\n")

from hifitime import Epoch, TimeSeries, Unit

if __name__ == "__main__":
    """
    The purpose of this script is to plot the differences between time systems.

    It will plot the difference between TAI, UTC, and all of the other timescales supported by hifitime.
    Then, as a separate plot, it will remove the UTC line to make the difference between other timescales more evident.
    """
    # Start by building a time series from 1970 until 2023 with a step of 30 days.
    ts = TimeSeries(
        Epoch("1970-01-01 00:00:00 UTC"),
        Epoch("2023-01-01 00:00:00 UTC"),
        Unit.Day * 30.0,
        True,
    )

    # Define the storage array
    data = []
    # Define the columns
    columns = [
        "UTC Epoch",
        "Δ TT (s)",
        "Δ ET (s)",
        "Δ TDB (s)",
        "Δ UTC (s)",
        "ET-TDB (s)",
    ]

    for epoch in ts:
        delta_utc = epoch.to_utc_duration() - epoch.to_tai_duration()
        delta_tt = epoch.to_tt_duration() - epoch.to_tai_duration()
        delta_tdb = epoch.to_tdb_duration_since_j1900() - epoch.to_tai_duration()
        delta_et = epoch.to_et_duration_since_j1900() - epoch.to_tai_duration()
        delta_et_tdb = delta_et - delta_tdb
        # Convert the epoch into a pandas datetime
        pd_epoch = pd.to_datetime(str(epoch))
        # Build the pandas series
        data.append(
            [
                pd_epoch,
                delta_tt.to_seconds(),
                delta_et.to_seconds(),
                delta_tdb.to_seconds(),
                delta_utc.to_seconds(),
                delta_et_tdb.to_seconds(),
            ]
        )

    df = pd.DataFrame(data, columns=columns)

    fig = px.line(
        df,
        x="UTC Epoch",
        y=columns[1:-1],
        title="Time scale deviation with respect to TAI",
    )
    fig.write_html("./target/time-scale-deviation.html")
    fig.show()

    fig = px.line(
        df,
        x="UTC Epoch",
        y=columns[1:-2],
        title="Time scale deviation with respect to TAI (excl. UTC)",
    )
    fig.write_html("./target/time-scale-deviation-no-utc.html")
    fig.show()

    fig = px.line(
        df,
        x="UTC Epoch",
        y=columns[-1],
        title="Time scale deviation of TDB and ET with respect to TAI",
    )
    fig.write_html("./target/time-scale-deviation-tdb-et.html")
    fig.show()



from hifitime import Epoch, TimeSeries, Unit
from datetime import datetime


def test_strtime():
    """
    Tests both strp and strftime
    """
    epoch = Epoch("2023-04-13 23:31:17 UTC")
    dt = datetime(2023, 4, 13, 23, 31, 17)

    epoch_fmt = epoch.strftime("%A, %d %B %Y %H:%M:%S")
    dt_fmt = dt.strftime("%A, %d %B %Y %H:%M:%S")

    assert epoch_fmt == dt_fmt

    assert Epoch.strptime(dt_fmt, "%A, %d %B %Y %H:%M:%S") == epoch


def test_utcnow():
    epoch = Epoch.system_now()
    dt = datetime.utcnow()

    # Hifitime uses a different clock to Python and print down to the nanosecond
    assert dt.isoformat()[:21] == f"{epoch}"[:21]


def test_time_series():
    """
    Time series are really cool way to iterate through a time without skipping a beat.
    """

    nye = Epoch("2022-12-31 23:59:00 UTC")

    time_series = TimeSeries(
        nye,
        nye + Unit.Second * 10,
        Unit.Second * 1,
        inclusive=True,
    )
    print(time_series)

    for num, epoch in enumerate(time_series):
        print(f"#{num}:\t{epoch}")

    assert num == 10
